library dipnav;

import 'package:flutter/widgets.dart';
import 'package:flutter_dipnav/src/dip_nav_data.dart';
import 'package:flutter_dipnav/src/dip_nav_route.dart';
import 'package:flutter_dipnav/src/dip_nav_router.dart';

export 'src/dip_nav_router.dart';
export 'src/dip_nav_route.dart';
export 'src/dip_nav_data.dart';

/// Extension to get [DipNavRouter] from the [BuildContext].
extension DipNavExtension on BuildContext {
  DipNavRouter get dipNav {
    return DipNav.of(this).router;
  }

  DipNavData? get dipNavData {
    final args = ModalRoute.of(this)?.settings.arguments;
    if (args != null && args is Map) {
      return DipNavData.fromMap(args as Map<String, dynamic>);
    }
    return null;
  }
}

/// The widget that propagate [DipNavRouter] down the tree.
class DipNav extends InheritedWidget {
  /// {@macro flutter_dipnav.DipNavRouter}
  final DipNavRouter router;

  const DipNav({Key? key, required this.router, required Widget child})
      : super(key: key, child: child);

  factory DipNav.builder({
    Key? key,
    required DipNavRoute initRoute,
    required Widget Function(DipNavDelegate delegate, DipNavParser parser,
            DipNavDispatcher dispatcher)
        builder,
    DipNavRoute? notFoundRoute,
    List<DipNavRoute>? routes,
  }) {
    final dipNav = DipNavRouter(
        initRoute: initRoute, notFoundRoute: notFoundRoute, routes: routes);
    return DipNav(
      key: key,
      router: dipNav,
      child: Builder(
        builder: (context) =>
            builder(dipNav.delegate, dipNav.parser, dipNav.dispatcher),
      ),
    );
  }

  static DipNav of(BuildContext context) {
    final DipNav? result = context.dependOnInheritedWidgetOfExactType<DipNav>();
    assert(result != null, 'No DipNav found in context');
    return result!;
  }

  @override
  bool updateShouldNotify(covariant DipNav oldWidget) {
    return router != oldWidget.router;
  }
}
