part of 'dip_nav_router.dart';

/// A delegate that is used by the [Router] widget to build and configure a
/// navigating widget.
class DipNavDelegate extends RouterDelegate<DipNavRoute>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<DipNavRoute> {
  /// The list of [ResultablePage]
  final List<_DipNavResultablePage> _stackPages = [];

  /// The list of [DipNavRoute]
  final List<DipNavRoute> routes;

  /// The [DipNavRoute]
  final DipNavRoute initRoute;

  /// The [DipNavRoute]
  final DipNavRoute? notFoundRoute;

  DipNavDelegate(
      {required this.initRoute, this.notFoundRoute, this.routes = const []}) {
    setInitialRoutePath(initRoute);
  }

  /// Called by the [Router] at startup with the structure that the
  /// [RouteInformationParser] obtained from parsing the initial route.
  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navigatorKey,
      onPopPage: _onPopPage,
      pages: List.of(_stackPages),
    );
  }

  @override
  GlobalKey<NavigatorState> get navigatorKey => GlobalKey<NavigatorState>();

  @override
  Future<void> setNewRoutePath(DipNavRoute configuration) {
    _stackPages.clear();
    return pushRoute(configuration);
  }

  /// Called by the [Router] when it detects a route information may have
  /// changed as a result of rebuild.
  @override
  DipNavRoute get currentConfiguration {
    final route = _stackPages.last.route;
    return route;
  }

  Future<T?> pushRoute<T>(DipNavRoute route,
      {bool removeUntil = false, bool replaceCurrent = false}) async {
    assert(!(removeUntil && replaceCurrent),
        'Only removeUntil or replaceCurrent should by true!');

    if (removeUntil) {
      _stackPages.clear();
    } else if (replaceCurrent && _stackPages.isNotEmpty) {
      _stackPages.removeLast();
    }
    _addPageData(route);
    notifyListeners();
  }

  @override
  Future<bool> popRoute() {
    if (_stackPages.length > 1) {
      _removePage(_stackPages.last);
      return Future.value(true);
    }
    return Future.value(false);
  }

  bool _onPopPage(Route<dynamic> route, dynamic result) {
    final didPop = route.didPop(result);
    if (!didPop) {
      return false;
    }
    _stackPages.remove(route.settings);
    notifyListeners();
    return true;
  }

  void _removePage(_DipNavResultablePage page) {
    _stackPages.remove(page);
    notifyListeners();
  }

  _DipNavResultablePage _createPage(DipNavRoute route) {
    return _DipNavResultablePage(
      route: route,
    );
  }

  void _addPageData(DipNavRoute route) {
    _stackPages.add(
      _createPage(route),
    );
  }
}
