import 'package:flutter/material.dart';
import 'package:flutter_dipnav/flutter_dipnav.dart';
import 'package:flutter_dipnav/src/dip_nav_route.dart';

part 'dip_nav_delegate.dart';
part 'dip_nav_dispatcher.dart';
part 'dip_nav_parser.dart';
part 'dip_nav_resultable_page.dart';

/// Main class of routing based on Navigator 2.0.
class DipNavRouter {
  late DipNavDelegate _delegate;
  late DipNavParser _parser;
  late DipNavDispatcher _dispatcher;

  /// {@macro flutter_dipnav._DipNavDelegate}
  DipNavDelegate get delegate => _delegate;

  /// {@macro flutter_dipnav._DipNavParser}
  DipNavParser get parser => _parser;

  /// {@macro flutter_dipnav._DipNavDispatcher}
  DipNavDispatcher get dispatcher => _dispatcher;

  /// {@macro flutter_dipnav._DipNavRouter.routes}
  List<DipNavRoute> get routes => List.of(_delegate.routes);

  /// {@macro flutter_dipnav._DipNavRouter.initRoute}
  DipNavRoute get initRoute => _delegate.initRoute;

  /// {@macro flutter_dipnav._DipNavRouter.notFoundRoute}
  ///
  /// [routes] is a list of routes to parse from URL change in the browser
  /// no need to add initialization and not found routes to this list
  DipNavRoute? get notFoundRoute => _delegate.notFoundRoute;

  DipNavRouter(
      {required DipNavRoute initRoute,
      DipNavRoute? notFoundRoute,
      List<DipNavRoute>? routes}) {
    bool isFoundInitRoute = false;
    bool isFoundNotFoundRoute = false;
    final routesList = List.of(routes ?? <DipNavRoute>[]);
    for (var route in routes ?? <DipNavRoute>[]) {
      if (route.path == initRoute.path) {
        isFoundInitRoute = true;
      }
      if (notFoundRoute != null && route.path == notFoundRoute.path) {
        isFoundNotFoundRoute = true;
      }
      if (isFoundInitRoute && (notFoundRoute == null || isFoundNotFoundRoute))
        break;
    }
    if (!isFoundInitRoute) {
      routesList.add(initRoute);
    }
    if (notFoundRoute != null && !isFoundNotFoundRoute) {
      routesList.add(notFoundRoute);
    }
    _delegate = DipNavDelegate(
      initRoute: initRoute,
      notFoundRoute: notFoundRoute,
      routes: routesList,
    );
    _parser = DipNavParser(this);
    _dispatcher = DipNavDispatcher(this);
  }

  /// Push method of adding new route
  Future<T?> push<T>(
      {String? key,
      required String path,
      Map<String, dynamic>? queries,
      required Widget page,
      final Object? arguments,
      bool removeUntil = false,
      bool replaceCurrent = false}) async {
    return _delegate.pushRoute(
        DipNavRoute(
          key: key,
          path: path,
          page: page,
          queries: queries,
          arguments: arguments,
        ),
        removeUntil: removeUntil,
        replaceCurrent: replaceCurrent);
  }

  /// Pop method for return to previous route
  Future<bool> pop<T>({T? arguments}) {
    return _delegate.popRoute();
  }
}
